<?php
/**
 *	Line In Pages Base Class
 *
 *	@version 0.3.0
 *	@author	Simon Fairbairn 
 */


if ( !class_exists('LIBC_Pages') ) {
	
	abstract class LIBC_Pages  {
		/**
		 * A way to keep track of whether this plugin is being used and only loading scripts on 
		 * those pages
		 * @access 	static
		 * @var 	boolean
		 */		
		static 		$add_script;	
		protected	$options = array();
		protected 	$options_page = null;
		protected 	$custom_parent_page = false;
		protected	$self = null;
		private		$menu_page_boxes = null;
		private		$submenu_page_boxes = null;
		private		$_standard_meta_boxes = null;
		private		$submenu_count = 0;
		/**
		 * The URL of the plugin
		 * @access 	protected
		 * @uses	plugins_url()
		 * @var 	string
		 */		
		protected 	$plugin_url = null;
		/**
		 * The plugin basename
		 * @access 	protected
		 * @uses	plugin_basename()
		 * @var 	string
		 */		
		protected 	$plugin_basename;
		/**
		 * The url of the scripts for this plugin
		 * @access 	protected
		 * @uses	Line_In_Typography::$plugin_url
		 * @var 	string
		 */		
		protected	$scripts_url;
		/**
		 * The url of the stylesheet for this plugin
		 * @access 	protected
		 * @uses	Line_In_Typography::$plugin_url
		 * @var 	string
		 */		
		protected	$styles_url;
		/**
		 * An arbitrary id identifier for this plugin.
		 * @access	protected
		 * @var		string
		 */ 
		protected 	$plugin_id;
		/**
		 * The name of the options key to be stored in the wp_options table.
		 * @access 	protected
		 * @uses	Line_In_Typography::$plugin_id
		 * @var 	string
		 */		
		protected	$options_name;
		/**
		 * The parent page that a submenu page is attached to. Needs to be set if using 
		 * @access 	protected
		 * @var 	string
		 */		
		protected 	$parent_page = null;
		/**
		 * The title for the plugin, to be used as a heading on the Options Page. 
		 * @access 	protected
		 * @usedby	LIPBC_Settings_API_With_Page::add_submenu_page()
		 * @var 	string
		 */		
		protected 	$plugin_title;
		/**
		 * An abbreviation of the title to be used on the menu itself
		 * @access 	protected
		 * @var 	string
		 */		
		protected 	$plugin_shortname;
		/**
		 * The lanugage identifier to be used when creating translations
		 * @access 	protected
		 * @var 	string
		 */		
		protected 	$lang;
		/**
		 * The version of the plugin, taken from the constant set at the top of the page
		 * @access 	protected
		 * @var 	array
		 */		
		protected 	$ver;
		/**
		 * Can be populated with an array of meta boxes, or left null if you don't want to add any
		 * @access 	protected
		 * @var 	array
		 */		
		protected 	$meta_boxes = null;
		/**
		 * Whether or not to show the settings link on the Plugins page.
		 * @access 	protected
		 * @var 	boolean
		 */		
		protected	$show_settings_link = true;
		/**
		 * Can be populated with an array menu page options 
		 * @access 	protected
		 * @var 	array
		 */		
		private 	$menu_page = null;
		/**
		 * Can be populated with an array of multiple submenu page options 
		 * @access 	protected
		 * @var 	array
		 */		
		protected 	$submenu_pages = null;
		/**
		 * Whether or not to add the sideboxes (about, etc.)
		 * @access 	protected
		 * @var 	array
		 */		
		protected 	$add_sideboxes = true;
	
		function __construct() {
			$class = $this->self;
				
			$this->plugin_id 		= $class::PLUGINID;			
			$this->plugin_title 	= $class::TITLE;
			$this->plugin_shortname = $class::SHORTNAME;
			$this->parent_page		= $class::PARENT;
			$this->add_sideboxes	= $class::SIDEBOXES;
			
			
			
			
			
			if ( method_exists($this, '_setup_plugin') ) {
				$this->_setup_plugin();
			} else {
				wp_die('The class that extends LIBC_Pages must have a method _setup_plugin');
			}
			if ( $this->self == null ) {
				wp_die("<p>Your extending class must define itself using <code>\$this->self = __CLASS__</code>. If I had late static bindings, you wouldn't have to do this.</p> <p>But I don't.</p> <p>So you do.</p>");
			}
			if ( $this->plugin_url == null ) {
				wp_die("Your extending class must define it's url using <code>\$this->plugin_url = plugins_url('', __FILE__)</code> or by using <code>get_theme_directory_uri()</code>");
			}
						
 			



			
			$this->plugin_nonce		= $this->plugin_id . '_nonce';
			$this->options_name 	= $this->plugin_id . '_options';
			$this->scripts_url 		= $this->plugin_url . 'js/';
			$this->styles_url 		= $this->plugin_url . 'css/';
			

			if ( is_admin() ) {
				if ( $this->menu_page !== null ) {
					$this->custom_parent_page = true;
					add_action('admin_menu', array( &$this, 'add_menu_page' ) );
				}
				if ( $this->submenu_pages !== null ) {
					add_action('admin_menu', array( &$this, 'add_submenu_pages' ) );
				}
				if ( $this->show_settings_link ) {

					if ( $this->plugin_basename != '' ) {
						add_filter("plugin_action_links_" . $this->plugin_basename, array( &$this, 'add_settings_link' ) );
					} 
				} 
				
				// For standard post/page meta boxes
				add_action( 'save_post', array( &$this, 'save_postdata' ) );
				add_action( 'add_meta_boxes', array(&$this, 'load_meta_boxes') );
			}
		}
		
		/**
		 *	Check to see if the options exisit in the database. If not, get the defaults
		 *	and load them in.
		 */
		public function add_settings_link($links) { 
			if ( $this->custom_parent_page ) {
				$settings_link = '<a href="admin.php?page=' . $this->plugin_id . '">Settings</a>';
			} else {
				$settings_link = '<a href="options-general.php?page=' . $this->plugin_id . '">Settings</a>'; 
			}
			array_unshift($links, $settings_link); 
			return $links; 
		}
		
		
		protected function _set_menu_page( $options, $boxes = false ) {
		
			$this->menu_page = $options;
			if ( $boxes )
				$this->menu_page_boxes = $boxes;
		}
		
		protected function _set_submenu_page( $options, $boxes = false) {
			$this->submenu_pages[$this->submenu_count] = $options;
			if ( $boxes ) 
				$this->submenu_page_boxes[$this->submenu_count] = $boxes;

			$this->submenu_count++;
		}
		
		/* Menu Pages */
		public function add_menu_page() {
		
			if ( isset ( $this->menu_page['title'] ) ) {
				$title = $this->menu_page['title'];
			} else {
				$title = $this->plugin_title;
			}
			
			$options_page = add_menu_page( $title, $title, 'manage_options', $this->plugin_id, array( &$this, 'parent_page'), $this->menu_page['icon_url'], $this->menu_page['position']);		
			$this->parent_page = $this->plugin_id;
			
			if ( $this->menu_page_boxes !== null ) { 
				foreach ( $this->menu_page_boxes as $key => $box  ) {
					$this->menu_page_boxes[$key]['page'] = $options_page;
				}
			}
			
			
			add_action('admin_print_styles-' . $options_page, array( &$this, 'enqueue_styles') );
			add_action('admin_print_styles-widgets.php', array( &$this, 'enqueue_styles') );
			add_action('admin_print_styles-' . $options_page, array( &$this, 'enqueue_scripts') );		
					
			add_action('load-' . $options_page, array( &$this, 'load_menu_meta_boxes' ) );
				
			if ( isset( $this->menu_page['help_callback'] ) ) {
				$method = $this->menu_page['help_callback'];
				add_contextual_help( $options_page, $this->$method()  );			
			}
			$this->selected_page[$this->plugin_id] = $options_page;

			
		}
		
		
		
		
		public function add_submenu_pages() {
			$pages = 0;
			foreach( $this->submenu_pages as $submenu_key => $page ) {
				if ( isset ( $page['special'] ) && $page['special' ] == 'theme' ) {
					$page_slug = add_theme_page(
						__( $page['title'], $this->lang ),   // Name of page
						__( $page['title'], $this->lang),   // Label in menu
						'edit_theme_options',                    // Capability required
						'theme_options',                         // Menu slug, used to uniquely identify the page
						array(&$this, $page['page_callback']) // Function that renders the options page
					);
				} else {

					$page_slug = add_submenu_page(
						$this->parent_page, 
						$page['title'], 
						$page['title'], 
						'manage_options', 
						$page['id'], 
						array( &$this, $page['page_callback']) 
					);
				}
				
				// If there are meta boxes any meta boxes for this page, get them configured to this page
				if ( isset( $this->submenu_page_boxes[$submenu_key] ) && sizeof( $this->submenu_page_boxes[$submenu_key] > 0 ) ) {				
					foreach ( $this->submenu_page_boxes[$submenu_key] as $key => $box  ) {

						$this->submenu_page_boxes[$submenu_key][$key]['page'] = $page_slug;

						
						$this->submenu_meta_boxes[$page['id']][] = $this->submenu_page_boxes[$submenu_key][$key];
						
						
					}
				}
				
				add_action('load-' . $page_slug, array( &$this, 'load_submenu_meta_boxes' ) );
				
				add_action('admin_print_styles-' . $page_slug, array( &$this, 'enqueue_styles') );
				add_action('admin_print_styles-widgets.php', array( &$this, 'enqueue_styles') );
				add_action('admin_print_styles-' . $page_slug, array( &$this, 'enqueue_scripts') );		

				if ( false !== $page['help_callback']  ) {
					$method = $page['help_callback'];
					add_contextual_help( $page_slug, $this->$method()  );			
				}
				$this->selected_page[$page['id']] = $page_slug;

			}
		}
		
		
		
		/* Meta boxes */
		
		public function load_menu_meta_boxes() {
			if ( $this->menu_page_boxes != null ) {
				wp_enqueue_script('common');
				wp_enqueue_script('wp-lists');
				wp_enqueue_script('postbox');
				foreach ( $this->menu_page_boxes as $meta_box ) {
					add_meta_box( $meta_box['id'], $meta_box['title'], array( &$this, $meta_box['callback'] ), $meta_box['page'], $meta_box['position'], 'default' );
				}
			}
		}
				
		public function load_submenu_meta_boxes() {
			if ( isset( $_GET['page'] ) ) {
				$current_page = $_GET['page'];
			} else {
				return false;
			}

			
			if ( isset( $this->submenu_meta_boxes[$current_page] ) ) {			
				wp_enqueue_script('common');
				wp_enqueue_script('wp-lists');
				wp_enqueue_script('postbox');
				foreach ( $this->submenu_meta_boxes[$current_page] as $meta_box ) {
					add_meta_box( $meta_box['id'], $meta_box['title'], array( &$this, $meta_box['callback'] ), $meta_box['page'], $meta_box['position'], 'default' );
				
				}
			}
		}
		
		protected function _add_meta_box( $options ) {
			$this->_standard_meta_boxes[] = $options;
		}
		
		public function load_meta_boxes() {
		
			if ( $this->_standard_meta_boxes !== null ) {
				foreach ( $this->_standard_meta_boxes as $meta_box ) {
					add_meta_box( $meta_box['id'], $meta_box['title'], array( &$this, $meta_box['callback'] ), 	$meta_box['page'], $meta_box['position'], 'default' );
				}
			}
			

			
		}
		
		protected function _all_meta() {
			echo "<style>.postbox .inside h3 {background: none; border: 0; box-shadow: none; -moz-box-shadow: none; -webkit-box-shadow: none; -o-box-shadow: none; -ms-box-shadow: none; }</style>";
		}
		
		
		public function about_meta_html() {
			
			// This is where the default text for the About meta will go
			echo "
			<h3>My Dear Friends,</h3>";
			echo "<p class='alignright' style='margin-left: 8px'>" . get_avatar( 'simon@line-in.co.uk', $size = '96', $default = '<path_to_url>' ) . "</p>"; 
			echo "<p>My name is Simon Fairbairn and I have the great privilege of being employed as Chief Pixel Engineer and Executive Code Manipulator over at <a href='http://line-in.co.uk'>Line In Web Design</a>.</p>
			
			<p>Truly, it is a great time to be alive, what with all the fascinating new inventions being developed almost, it seems, on a daily basis. We have the motor car, electric light bulbs and (if I may be so bold, my current favourite) the Inter Web Network</p>
			
			<p>Of course, such major leaps in technology bring with them certain difficulties, which is why we here at <a href='http://line-in.co.uk'>Line In Web Design</a> have created a range of plugins and themes that we sincerely hope may assist you in navigating these strange and exciting new waters. </p>
			
			<p>We pray that they prove to be useful to you, however, should you have any comments or suggestions then please don't hesitate to send me a digital telegraph to <a href='mailto:simon@line-in.co.uk'>simon@line-in.co.uk</a>.</p> 
			
			<p>And should you ever need any help with any sort of WordPress design or development, we are at your service.</p>
			
			<p>Yours With Deepest Regards,</p>
			
			<h3>Simon A. Fairbairn Esq.</h3>
			
			
				";
		}
		
		/* For meta boxes on regular posts and pages */
		
		public function pre_save( $post_id ) {
			// verify if this is an auto save routine. 
			// If it is our form has not been submitted, so we dont want to do anything
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) 
				return false;

			if ( !isset( $_POST[$this->plugin_nonce  . '_' . $post_id] ) ) {
				
				return false;
			}
			
			if ( !wp_verify_nonce( $_POST[$this->plugin_nonce . '_' . $post_id], $this->plugin_id ) ) {

				return false;
			}
			  // Check permissions
			if ( !current_user_can( 'edit_post', $post_id ) ) 
				return false;

			return true;
			
		}
		
		
		/* Scripts and styles */
			
		
		public function load_scripts( $file_name, $deps = false ) {

			if ( WP_DEBUG ) {
				$ver = time();
			} else {
				$ver = $this->ver;
			}
			
			if ( !isset( $this->scripts_url ) ) {
				$this->scripts_url = $this->plugin_url . 'js/';
			}
			
		
			wp_register_script( $this->plugin_id . '-scripts', $this->scripts_url  . $file_name . '.js', $deps, $ver, true );
		}
		public function load_styles( $file_name, $deps = false ) {
			if ( WP_DEBUG ) {
				$ver = time();
			} else {
				$ver = $this->ver;
			}
			if ( !isset( $this->styles_url ) ) {
				$this->styles_url = $this->plugin_url . 'css/';
			}
			wp_register_style( $this->plugin_id . '-styles', $this->styles_url  . $file_name . '.css', $deps, $ver, 'all');
		}
		
		public function enqueue_scripts() {
		
			wp_enqueue_script( $this->plugin_id . '-scripts' );
		}
		
		public function enqueue_styles() {
			wp_enqueue_style( $this->plugin_id . '-styles' );
		}
		

		/* HTML Output */
		
		public function parent_page() {
			$this->page_html();
		}
		
		public function page_html( $title = false, $callback = false ) {
			$page_id = $_GET['page'];
			$page = $this->selected_page[$page_id];

			?>
			<div class='wrap'>
				<?php screen_icon(); ?>
				<?php if ( $title ) { ?>

					<h2><?php _e( $title,  $this->lang ); ?></h2>				
				<?php } else { ?>

					<h2><?php _e( $this->plugin_title,  $this->lang ); ?></h2>
				<?php } ?>
				<?php 
				
				if ( true == $this->add_sideboxes ) {
					add_meta_box( $this->plugin_id . '-about', 'A Warm Letter To Our Users', array(&$this, 'about_meta_html'), $page, 'side', 'core');
					
				}
				
				/*
					
					if ( isset( $_GET['settings-updated'] ) ) {
					    echo "<div class='updated'><p>Settings updated successfully.</p></div>";
					}
					*/
				 ?>
				 <form action="options.php" method="post">
					 <?php wp_nonce_field('closedpostboxes', 'closedpostboxesnonce', false ); ?>
					 <?php wp_nonce_field('meta-box-order', 'meta-box-order-nonce', false ); ?>
					
						<div id="poststuff" class="metabox-holder<?php echo ' has-right-sidebar'; ?>">
							<div id="side-info-column" class="inner-sidebar">
								<?php  do_meta_boxes($page, 'side', null); ?>
							</div>
							<div id="post-body" class="has-sidebar">
								<div id="post-body-content" class="has-sidebar-content">
									<?php do_meta_boxes($page, 'normal', null); ?>
								</div>
							</div>
					 <p class='submit clear'><?php submit_button('Update options', 'primary','submit', false); ?> <?php submit_button('Reset options', 'secondary', 'reset', false); ?></p>

				 </form>
			</div>
			<script type="text/javascript">
				//<![CDATA[
				jQuery(document).ready( function($) {
					// close postboxes that should be closed
					$('.if-js-closed').removeClass('if-js-closed').addClass('closed');
					// postboxes setup
					postboxes.add_postbox_toggles('<?php echo $page; ?>');
				});
				//]]>
			</script>
			 <?php
		}


		abstract function save_postdata( $post_id );
	}
}


?>