<?php
/**
 *	Line In Settings API base class
 *
 *	@version 0.3.0
 */
 
if ( !class_exists( 'LIBC_Settings_API' ) ) {
	class LIBC_Settings_API  {
		static 		$current_group = false;
		private		$_run_settings_fields = true;
		static		$group_count = 1;
		private		$group;
		private 	$_options_name;
		private		$_field_count = 0;
		protected 	$_settings_object;
		
		private		$_loaded_section = 0;
		

		function __construct( LI_Settings_Base $obj ) {
	
			if ( !$obj instanceof LI_Settings_Base ) {
				wp_die("You need to pass an object that extends the LI_Settings_Base when instantiating the LIBC_Settings_API class");
			}
			$this->_settings_object = $obj;
			
			/**
			 *	Step 1. Register the settings, using the passed id as the constant reference.
			 */
			
			add_action('admin_init', array( &$this, 'register_settings') );

		}
		
		private function _get_section() {
			return $this->_settings_object->get_option_group() . '_section_' . $this->group;
		}
		
		private function _get_page() {
			return $this->_settings_object->get_option_group() . '_page_' . $this->group;
		}
	
		/**
		 *	Initial settings register
		 */
		public function register_settings() {
			/* Settings Group, Options Name (db), Validation callback */
			register_setting( 
				$this->_settings_object->get_option_group(), 
				$this->_settings_object->get_options_name(), 
				array( $this->_settings_object, 'validate' ) 
			);
			$this->add_sections();
			
		}
		
		public function add_sections() {
		
			while ( $this->_settings_object->has_settings_sections() ) {

				$this->_registered_sections[] = $this->_settings_object->get_section_id() . '_page' ;
				
				
				add_settings_section( 
					$this->_settings_object->get_section_id(), 
					$this->_settings_object->get_section_title(), 
					array( $this->_settings_object, $this->_settings_object->get_section_callback() ), 
					$this->_settings_object->get_section_id() . '_page'
				);
				
				$this->add_settings();
				
			}
			
		}
		
		public function add_settings() {

			while ( $this->_settings_object->has_settings() ) {
				$label = false;
				if ( $this->_settings_object->get_setting_option('label') ) {
					$label = array( 'label_for' => $this->_settings_object->get_setting_option('label') );
				}
	
				add_settings_field( 
				
					$this->_settings_object->get_setting_option('id'), 
					$this->_settings_object->get_setting_option('title'), 
					array( $this->_settings_object, $this->_settings_object->get_setting_option('callback') ),
					$this->_settings_object->get_section_id()  . '_page', 
					$this->_settings_object->get_section_id(),
					$label
				);
				$this->_field_count++;
			}
		}
		
			
		protected function _get_default_options() {
			return $this->_settings_object->get_defaults();
		}
			
		public function get_settings( $section = false ) {

			if ( $this->_run_settings_fields ) {
				$this->_run_settings_fields = false;
				settings_fields($this->_settings_object->get_option_group());
			}	
			do_settings_sections( $this->_registered_sections[$this->_loaded_section]  );
			$this->_loaded_section++;
		}
		
		public function get_settings_errors() {
			 settings_errors();
		}
		
		public function get_options() {
			return $this->_settings_object->get_option();
		}
		public function get_option( $name = false) {
			if ( !$name )
				return $false;
			else 
				return $this->_settings_object->get_option( $name );
		}
		
	}
}

if ( !class_exists('LI_Settings_Base') ) {
	abstract class LI_Settings_Base  {
		
		// The id for the overall settings group
		protected	$_option_group		= null;
		// The options name that will be used as the key in the options database. Once per page.
		protected	$_options_name		= null;
		// The options themselves from the database (or defaults if this is your first time)
		protected	$_options			= false;
		// A counter to see how many groups we have
		private		$_current_group		= null;
		// A counter for the has_settings_section() method
		private		$_current_section	= 0;
		// An array of groups
		private		$_groups 			= null;
		
		protected	$_current_setting 	= 0;
		protected	$_settings 			= null;
		protected	$_allowed_keys 		= null;
		
		
		
		
		public function __construct( $plugin_id = false ) {
			
			// Set the ID for this entire page's settings
			$this->_set_option_group( $plugin_id );
			
			if ( $this->self == null ) {
				wp_die("<p>Your extending class must define itself using <code>\$this->self = __CLASS__</code>. If I had late static bindings, you wouldn't have to do this.</p> <p>But I don't.</p> <p>So you do.</p>");
			}
			
			$self = $this->self;
			$this->lang = $self::LANG;
						

			// Set the options name for this entire page's settings
			$this->_set_options_name();
			$this->_allowed_keys = array( 
										'id',
										'title',
										'callback',
										'label'
									);
			$this->load_settings();
			// Take care of the default options
			add_action('init', array( &$this, 'options_init'), 9 );
			
		}
		
		protected function _set_option_group( $id ) {
			if ( $id ) {
				$this->_option_group = $id;
			} else {
				$self = $this->self;
				$this->_option_group = $self::PLUGINID;
			}

		}
		protected function _set_options_name() {
			$this->_options_name = $this->_option_group . '_options';
		}
		protected function new_group($title = false, $id = false, $intro_callback = false) {
			if ( $this->_current_group === null ) {
				$this->_current_group = 0;
			} else {
				$this->_current_group++;
			}
			if ( $id === false ) {
				$id = $this->get_option_group() . '_' . $this->_current_group;
			}
			
			$this->_groups[$this->_current_group] = array(
				'id' 		=> $id,
				'title' 	=> $title,
				'callback' 	=> $intro_callback
			);
		}
		
		public function get_section_id() {
			$section = $this->_current_section - 1;
			return $this->_option_group . '_' . $this->_groups[$section]['id'];
		}
		
		public function get_section_title() {
			$section = $this->_current_section - 1;
			return $this->_groups[$section]['title'];			
		}
		
		public function get_section_callback() {
			$section = $this->_current_section - 1;		
			return $this->_groups[$section]['callback'];
		}
		
				
		public function has_settings_sections() {
			if ( sizeof( $this->_groups ) > 0 ) {
				if ( isset( $this->_groups[$this->_current_section] ) ) {
					$this->_current_section++;
					$this->_current_setting = 0;
					return true;
				} else {
					return false;
				}
			}
			return false;
		}
		
		public function has_settings() {
			$section = $this->_current_section - 1;
			if ( isset( $this->_groups[$section]['settings']) && is_array( $this->_groups[$section]['settings']) ) {

				if ( isset( $this->_groups[$section]['settings'][$this->_current_setting] ) ) {
					$this->_current_setting++;
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}
		
		
		
		
		protected function add_setting( $id , $title = false, $label = true ) {
			if ( $this->_groups === null ) {
				$this->new_group();
			}
			$this->_groups[$this->_current_group]['settings'][] = array(
				'id' => $id,
				'callback' => $id,
				'title' => $title, 
				'label' => $label
			);
		}
		
		
		
		
		public function get_options_name( $name = false ) {
			if ( $name ) {
				return $this->_options_name . "[$name]";
			} else {
				return $this->_options_name;
			}
			
		}
				
		public function get_option_group( $field = false ) {

			if ( $field ) {
				return $this->_options_name . "-$field";
			} else {
				return $this->_option_group;
			}
			
			
		}
		
		public function get_name( $name = false ) {
			if ( $name ) {
				return $this->_options_name . "[$name]";
			} else {
				return $this->_options_name;
			}
			
		}
				
		public function get_id( $field = false ) {
			if ( $field ) {
				return $this->_options_name . "-$field";
			} else {
				return $this->_plugin_id;
			}
			
			
		}
		
		public function get_option( $name = false ) {

			
			if ( $name ) {
				if ( isset( $this->_options[$name] ) ) {
					return $this->_options[$name];
				} else {
					return false;
				}
			} else {
				return $this->_options;
			}
		
		}
				
		
		
		public function get_setting_option( $key = false ) {
			$section = $this->_current_section - 1;
		
			if ( in_array( $key, $this->_allowed_keys ) ) {
				$actual_setting = $this->_current_setting - 1;
				if ( isset( $this->_groups[$section]['settings'][$actual_setting][$key] ) ) {

					return $this->_groups[$section]['settings'][$actual_setting][$key];
					
				} else {
					return false;
				}	
			} else {
				return false;
			}
			
		}
		/**
		 *	Check to see if the options exisit in the database. If not, get the defaults
		 *	and load them in.
		 */
		public function options_init() {
		    // set options equal to defaults
		    $this->_options = get_option( $this->_options_name );
		    if ( false === $this->_options ) {

				$this->_options = $this->get_defaults();
	
		    }
		    update_option( $this->_options_name, $this->_options );
		}
	
		public function get_section_name() {
			if ( isset( $this->_section_name ) ) {
				return $this->_section_name;
			} else {
				return false;
			}
		}
	
		public function get_defaults() {
			return $this->defaults;
		}
		
		abstract function load_settings();
		abstract function validate( $input );
	}
	
	
}

?>