<?php
/*
Plugin Name: Line In Typography
Plugin URI: http://line-in.co.uk/
Description: This magical piece of markup monkery will make your mission to muster magnificent missives on your monitor much more manageable.
Author: Simon Fairbairn
Version: 0.3.3
Requires at least: 3.0
Author URI: http://line-in.co.uk/
License: GPL
*/
/**
 *	Imports the required base class full of plugin goodies
 */
include "classes/libc-pages.php"; 
include "classes/libc-settings-api.php"; 

interface LIT_Constants {
	const VERSION	= '0.3.3';
	const PLUGINID	= 'line_in_typography';
	const TITLE		= 'Line In Typography';
	const SHORTNAME	= 'LIT';
	const ABBR		= 'LIT';
	const PARENT	= 'options-general.php';
	const LANG		= 'lineintypography';
	const SIDEBOXES	= true;
}

/**
 *	Run on plugin activation. You can load your default options into the db at this point, but the Settings API
 *	will also take care of loading default options and it'll reduce redundancy if you only declare them once.
 */
register_activation_hook(__FILE__, array( 'Line_In_Typography', 'install_plugin' ) );

/**
 *	Run on plugin deactivation. 
 */
register_deactivation_hook(__FILE__, array( 'Line_In_Typography', 'uninstall_plugin') );

/**
 * 	Initiate the class
 *	@return	object	My_Test_Plugin object
 */
function call_Line_In_Typography() 
{
    return new Line_In_Typography();
}
add_action( 'init', 'call_Line_In_Typography' );

/**
 *  The main plugin class
 */
if ( !class_exists( 'Line_In_Typography' ) ) {
	class Line_In_Typography extends LIBC_Pages implements LIT_Constants {
		public $self = __CLASS__;
		public function _setup_plugin() {
			$this->plugin_basename	= plugin_basename(__FILE__);	
			$this->plugin_url		= plugins_url('', __FILE__) . '/';
			/** 
			 *	Prepare scripts  (load_scripts just registers them. They'll be 
			 *	enqueued as needed later).
			 *	Takes two paramaters: the first is required, and is the name of your JS file (no extension)
			 *	The second is an optional array of dependencies.
			 */
			$this->load_scripts('line-in-typography', array('jquery') );	
			
			if ( !is_admin() ) {
				$this->enqueue_scripts();
			}
				
			// Create classes (to use as a namespace) with all the HTML callbacks enclosed
			// Remember to pass the ID 
			$this->settings['page_1'] = new LIBC_Settings_API( new Settings_Page_1( 'li_typography' ) );

			$submenu_page_options = array(
				'id'		=> $this->plugin_id,
				'title' 	=> 'Line In Typography',
				'page_callback'	=> 'first_page',
				'help_callback'	=> '_help'
				
			);
			$submenu_page_box[] = array( 
				'id' 		=> 	'line-in-typography',
				'title'		=> 	'Line In Typography Settings',
				'callback'	=>	'load_meta',
				'position'	=>	'normal',
			);
			
			$this->_set_submenu_page( $submenu_page_options, $submenu_page_box );
			
			if ( !is_admin() ) {
				add_action('wp_head', array( &$this, 'head' ) );
				add_action('wp_footer', array( &$this, 'foot' ) );
			
			} 
			// END EXAMPLE STUFF
		}
		
		
		
		public function _help() {
			return "Contextual help coming soon";
		}	
				
		
		public function load_meta() {
			$this->_all_meta();
			$this->settings['page_1']->get_settings();
		}
	
			
			
		public function first_page() {
			
			$this->page_html( 'Line In Typography' );
			
		}
				
		static function install_plugin() {
			// Code for installation
		}
		
		static function uninstall_plugin() {
			// Code for deactivation
		}

		function head() { 
			self::$add_script = true; 
			
			?>
			<style>
				.gr960-12 {
					background: url(<?php echo $this->plugin_url; ?>css/img/960-12px.png) 0 -4px !important;
					background-size: 100% !important;
				}
				.gr960-16 {
					background: url(<?php echo $this->plugin_url; ?>css/img/960-16px.png) 0 -4px !important;
					background-size: 100% !important;
				}
				
				#lit-vertical-rhythm {
					width: 100%;
					z-index: 100;
					position: absolute;
					top: 0;
					left: 0;	
				}
				#lit-buttons {
					position: fixed;
					top: 50px;
					right: 0;
					z-index: 200;
					background: #fff;
					padding: 15px;
					border: 1px solid #000;
					-moz-border-radius: 10px;
					-webkit-border-radius: 10px;
					border-radius: 10px;
					width: 400px;
					
				}
				
				.switchImages embed, .switchImages img, .switchImages object, .switchImages video, .switchImages iframe  {
					display: none !important;	
				}
				
								
				/* =ENDDEL */
				            
            
            </style>
		<?php }
		
		
		function foot() { 
			if ( ! self::$add_script )
				return;
			wp_print_scripts($this->plugin_id . '-scripts'); 
			
			$options = $this->settings['page_1']->get_options();

			
		?>

			<script>
				/*
				 *	Variables:
				 *
				 *	containers are what the plugin is attached to
				 *
				 *	fontSizeContainer (default: body), for most of my sites it'll be wrap now
				 *	
				 *	Lines offset (default: 0), just in case you need to offset the lines
				 *
				 */
				jQuery(document).ready(function($) {
					var myOptions = {	
						pluginUrl			: "<?php echo $this->plugin_url; ?>",
						lineHeightContainer	: 'body',
						gridState			: 'on',
						lineState			: 'off',
						testHtml			: "<p><?php echo $options['paragraph-text']; ?></p>",
						showTestHtml		: '<?php echo $options['show-paragraph']; ?>',
						backgroundOffset	: <?php echo $options['background-offset']; ?>,
						lineHeight			: <?php echo $options['line-height']; ?>
					};
					$('<?php echo $options['elements']; ?>' ).LineInTypography(myOptions);
				});
			</script>

		<?php
		}
		
		public function save_postdata( $post_id ) {
		
		} 
	}
	
}

class Settings_Page_1 extends LI_Settings_Base implements LIT_Constants {
	public $self = __CLASS__;
	public function load_settings() {
		$this->defaults = array(
			'line-height' 		=> 21,
			'font-size'			=> 14,
			'elements'			=> '.container',
			'paragraph-text'	=> 'Paragraph to test line height',
			'show-paragraph'	=> 'true',
			'background-offset'	=> 0,
			'plugin-version'	=> $this->lang
		
		);
		
		$this->new_group(false, 'first_group', 'get_intro');
		$this->add_setting( 'line_height', 'Set Line Height' );
		$this->add_setting( 'elements', 'DOM Elements' );
		$this->add_setting( 'test_paragraph', 'Test Paragraph Text' );
		$this->add_setting( 'show_paragraph', 'Show Paragraph' );
		$this->add_setting( 'background_offset', 'Background Offset' );
	}
	
	/* Intro callbacks */


	public function get_intro() {
		
		?>
		 <p><?php _e( 'Manage options for the Line In Typography plugin. Refer to contextual help for more information.', $this->lang ); ?></p>
		<?php
		
	}

	public function line_height() {
		?>
		<input type='text' name='<?php echo $this->get_name('line-height'); ?>' value='<?php echo intval($this->get_option('line-height') ); ?>' id='<?php echo $this->get_id('line-height'); ?>' size='5' /> <span class='description'>Explicitly set your line height here, or set to 0 to have the plugin try and figure it out from your theme's body styles</span>
		
		<?php
	}
	function li_typography_font_size() {
		?>
		<input type='text' name='<?php echo $this->get_name('font-size'); ?>' value='<?php echo intval($this->get_option('line-height') ); ?>' id='<?php echo $this->get_id('font-size'); ?>' size='5' /> <span class='description'>Explicitly set your font size here, or set to 0 to have the plugin try and figure it out from your theme's body styles</span>
		<?php
	}
	function elements() {
		?>
		<input type='text' name='<?php echo $this->get_name('elements'); ?>' value='<?php echo esc_attr($this->get_option('elements') ); ?>' id='<?php echo $this->get_id('elements'); ?>' /> <span class='description'>Comma separated list of DOM elements to attach the grids to</span>
		<?php
	}
	function test_paragraph() {
		?>
		<input type='text' name='<?php echo $this->get_name('paragraph-text'); ?>' value='<?php echo esc_attr($this->get_option('paragraph-text')); ?>' id='<?php echo $this->get_id('paragraph-text'); ?>' size='50' /> <span class='description'>A test paragraph to show above all content. Indicates the correct positioning on the line that all paragraphs should adhere to.</span>
		<?php
	}
					
	
	function show_paragraph() {
		?>
		<input type="radio" name="<?php echo $this->get_name('show-paragraph'); ?>" value="true"<?php checked( 'true' == $this->get_option('show-paragraph') ); ?> id='<?php echo $this->get_id('show-paragraph-true'); ?>' /><label for='<?php echo $this->get_id('show-paragraph-true'); ?>'>Yes</label>
		<input type="radio" name="<?php echo $this->get_name('show-paragraph'); ?>" value="false"<?php checked( 'false' == $this->get_option('show-paragraph') ); ?> id='<?php echo $this->get_id('show-paragraph-false'); ?>' /><label for='<?php echo $this->get_id('show-paragraph-false'); ?>'>No</label>
		
		<?php
	}
	function background_offset() {
		?>
		<input type='text' name='<?php echo $this->get_name('background-offset'); ?>' value='<?php echo intval( $this->get_option('background-offset') ); ?>' id='<?php echo $this->get_id('background-offset'); ?>' size='5' /> <span class='description'>Shift the lines up or down to get your test paragraph sitting exactly on them.</span>
		
		<?php
	}

	function validate( $input ) {
		$valid_input = $this->get_option();
		$default_options = $this->get_defaults();
		
		if ( isset( $_POST['reset'] ) ) {
			$input = $default_options;
			add_settings_error('li_typography_settings_section', 'reset-message', "Options reset to default settings", 'updated');
		} else if ( isset( $_POST['submit'] ) ) {
			$error = false;
			if ( (int) $input['line-height'] < 0 ) {
				$input['line-height'] = $valid_input['line-height'];
				$error[] = "Line height not a valid positive integer, not updated";
			}
//				if ( (int) $input['font-size'] < 0 ) {
//					$input['font-size'] = $valid_input['font-size'];
//					$error[] = "Font size not a valid positive integer, not updated";
//				}
			$input['elements'] = wp_kses($input['elements'], array());
			if ( '' == $input['elements'] ) {
				$input['elements'] = $valid_input['elements'];
				$error[] = "You have not set a valid element to attach the grids to. Setting not updated.";	
				
			}
			
			
			$input['paragraph-text'] = wp_kses($input['paragraph-text'], array());
			$input['show-paragraph'] = ( 'true' == $input['show-paragraph'] ) ? 'true' : 'false';
			if ( 'true' == $input['show-paragraph'] && '' == $input['paragraph-text'] ) {
				$input['paragraph-text'] = $default_options['paragraph-text'];
				$error[] = "If you want to show the paragraph, you'll need some paragraph text. Set to default";
			}
			
			$input['background-offset'] = (int) $input['background-offset'];
			
			
			if ( false !== $error ) {
				$i = 0;
				$output = false;
				foreach ( $error as $err ) {
					
					if ( $i > 0 ) {
						$output .= "<br />";
					}
					$output .= $err;
					$i++;
				
				}
				add_settings_error('li_typography_settings_section', 'error-message', $output, 'error');
				add_settings_error('li_typography_settings_section', 'updated-message', "All other settings updated", 'updated');
			}
		}
		return $input;
	}
}



?>